@@ -52,6 +52,8 @@ enum {
 	PROP_RECORDERS_ONLY,
 };
 
+#define NAUTILUS_BURN_DRIVE_SELECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NAUTILUS_BURN_TYPE_DRIVE_SELECTION, NautilusBurnDriveSelectionPrivate))
+
 struct NautilusBurnDriveSelectionPrivate {
 	GList   *cdroms;
 	gboolean have_file_image;
@@ -106,6 +108,8 @@ nautilus_burn_drive_selection_class_init (NautilusBurnDriveSelectionClass *klass
 	object_class->get_property = nautilus_burn_drive_selection_get_property;
 	object_class->finalize = nautilus_burn_drive_selection_finalize;
 
+	g_type_class_add_private (klass, sizeof (NautilusBurnDriveSelectionPrivate));
+
 	/* Properties */
 	g_object_class_install_property (object_class, PROP_DEVICE,
 					 g_param_spec_string ("device", NULL, NULL,
@@ -188,7 +192,7 @@ nautilus_burn_drive_selection_init (NautilusBurnDriveSelection *selection)
 	GtkCellRenderer *cell;
 	GtkListStore    *store;
 
-	selection->priv = g_new0 (NautilusBurnDriveSelectionPrivate, 1);
+	selection->priv = NAUTILUS_BURN_DRIVE_SELECTION_GET_PRIVATE (selection);
 
 	store = gtk_list_store_new (1, G_TYPE_STRING);
 	gtk_combo_box_set_model (GTK_COMBO_BOX (selection),
@@ -221,12 +225,9 @@ nautilus_burn_drive_selection_finalize (GObject *object)
 		NautilusBurnDrive *cdrom = l->data;
 
 		l = g_list_remove (l, cdrom);
-		nautilus_burn_drive_free (cdrom);
+		nautilus_burn_drive_unref (cdrom);
 	}
 
-	g_free (selection->priv);
-	selection->priv = NULL;
-
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 	}
@@ -272,7 +273,7 @@ nautilus_burn_drive_selection_set_have_file_image (NautilusBurnDriveSelection *s
 
 		item = g_list_last (selection->priv->cdroms);
 		cdrom = (NautilusBurnDrive *)item->data;
-		nautilus_burn_drive_free (cdrom);
+		nautilus_burn_drive_unref (cdrom);
 		selection->priv->cdroms = g_list_delete_link
 			(selection->priv->cdroms, item);
 		gtk_widget_set_sensitive (GTK_WIDGET (selection), (selection->priv->cdroms != NULL));
@@ -339,7 +340,7 @@ nautilus_burn_drive_selection_set_recorders_only (NautilusBurnDriveSelection *se
 			      || drive->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER
 			      || drive->type & NAUTILUS_BURN_DRIVE_TYPE_FILE)) {
 				gtk_combo_box_remove_text (GTK_COMBO_BOX (selection), i);
-				nautilus_burn_drive_free (drive);
+				nautilus_burn_drive_unref (drive);
 				selection->priv->cdroms = g_list_delete_link (selection->priv->cdroms, l);
 			}
 
@@ -375,7 +376,7 @@ nautilus_burn_drive_selection_set_recorders_only (NautilusBurnDriveSelection *se
 									 drive,
 									 i);
 			} else {
-				nautilus_burn_drive_free (drive);
+				nautilus_burn_drive_unref (drive);
 			}
 		}
 		g_list_free (drives);
