@@ -61,6 +61,8 @@
 
 #ifdef USE_HAL
 #include <libhal.h>
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
 #endif /* USE_HAL */
 
 #include "nautilus-burn-drive.h"
@@ -104,11 +106,43 @@ typedef enum {
 	NAUTILUS_BURN_DRIVE_PROTOCOL_SCSI,
 } NautilusBurnDriveProtocolType;
 
-struct NautilusBurnDrivePriv {
+#define NAUTILUS_BURN_DRIVE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NAUTILUS_BURN_TYPE_DRIVE, NautilusBurnDrivePrivate))
+
+struct NautilusBurnDrivePrivate {
 	NautilusBurnDriveProtocolType protocol;
+
+	gboolean                      monitor_enabled;
+	guint                         poll_id;
+
+#ifdef USE_HAL
+	LibHalContext                *ctx;
+#endif
+
 	char                         *udi;
+
+	char                         *monitor_udi;
+	NautilusBurnDriveType         monitor_media_type;
+	gboolean                      monitor_tray_open;
 };
 
+/* Signals */
+enum {
+	MEDIA_ADDED,
+	MEDIA_REMOVED,
+	LAST_SIGNAL
+};
+
+enum {
+        PROP_0,
+        PROP_MONITOR_ENABLED
+};
+
+static int nautilus_burn_drive_table_signals [LAST_SIGNAL] = { 0 };
+
+static GObjectClass *parent_class = NULL;
+
+G_DEFINE_TYPE(NautilusBurnDrive, nautilus_burn_drive, G_TYPE_OBJECT);
+
 #ifdef __FreeBSD__
 
 #define get_ioctl_handle_fd(x) (((struct cam_device *)x)->fd)
@@ -165,13 +199,13 @@ get_hal_context (void)
 	if (ctx == NULL) {
 		ctx = libhal_ctx_new ();
 		if (ctx == NULL) {
-			g_warning ("Could not create a HAL context\n");
+			g_warning ("Could not create a HAL context");
 		} else { 
 			dbus_error_init (&error);
 			dbus_conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
 
 			if (dbus_error_is_set (&error)) {
-				g_warning ("Could not connect to system bus: %s\n", error.message);
+				g_warning ("Could not connect to system bus: %s", error.message);
 				dbus_error_free (&error);
 				return NULL;
 			}
@@ -179,8 +213,7 @@ get_hal_context (void)
 			libhal_ctx_set_dbus_connection (ctx, dbus_conn);
 
 			if (! libhal_ctx_init (ctx, &error)) {
-				g_warning ("Could not initalize "
-					   "the HAL context: %s\n",
+				g_warning ("Could not initalize the HAL context: %s",
 					   error.message);
 
 				if (dbus_error_is_set (&error))
@@ -631,22 +664,25 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 			int blank, rewrite, empty;
 
 			if (get_disc_status (fd, &empty, &rewrite, &blank) == 0) {
+				close_ioctl_handle (ioctl_handle);
 
 				if (is_rewritable)
 					*is_rewritable = rewrite;
-				
 				if (is_blank)
 					*is_blank = blank;
+
+				disc_type = get_disc_type (device);
+				disc_type_get_info (disc_type, is_blank, has_data, has_audio);
+
+				if (empty)
+					return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+				else
+					return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 			}
+
 			close_ioctl_handle (ioctl_handle);
 
-			disc_type = get_disc_type (device);
-			disc_type_get_info (disc_type, is_blank, has_data, has_audio);
-
-			if (empty)
-				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
-			else
-				return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
+			return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 		}
 	}
 
@@ -748,12 +784,12 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 	if (drive->priv != NULL && drive->priv->udi != NULL) {
 		LibHalContext        *ctx;
 		char                **device_names;
-		int                   num_devices;
+		int                   num_devices = -1;
 		NautilusBurnMediaType type;
 		char                 *hal_type;
 		DBusError             error;
 		
-		ctx = get_hal_context ();
+		ctx = drive->priv->ctx;
 
 		dbus_error_init (&error);
 		if (ctx != NULL) {
@@ -768,14 +804,14 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 				dbus_error_free (&error);
 				libhal_free_string_array (device_names);
 
-				goto fallback;
+				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 			}
 
-			if (num_devices == 0) {
+			if (num_devices <= 0) {
 				g_warning ("No HAL devices found for UDI '%s'", drive->priv->udi);
 				libhal_free_string_array (device_names);
 
-				goto fallback;
+				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 			}
 
 			/* just look at the first child */
@@ -848,16 +884,10 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 
 			libhal_free_string_array (device_names);
 
-			/* if the type has been determined using HAL, return it.
-			   Otherwise, fall through and try the non-HAL approach. */
-			if (type != NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN
-			    && type != NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
-				return type;
-			}
+			return type;
 		}
 	}
 
- fallback:
 	return nautilus_burn_drive_get_media_type_from_path_full (drive->device,
 								  is_rewritable,
 								  is_blank,
@@ -1287,6 +1317,133 @@ nautilus_burn_drive_eject (NautilusBurnDrive *drive)
 
 #ifdef USE_HAL
 
+static gboolean
+hal_udi_is_our_drive (LibHalContext     *ctx,
+                      const char        *udi,
+		      NautilusBurnDrive *drive)
+{
+	char    *device = NULL;
+	gboolean res    = FALSE;
+
+	if (drive == NULL || drive->device == NULL )
+		return res;
+
+	if (! libhal_device_property_exists (ctx, udi, "block.device", NULL))
+		return res;
+
+	device = libhal_device_get_property_string (ctx, udi, "block.device", NULL);
+	if (device && strcmp (device, drive->device) == 0) {
+		res = TRUE;
+	}
+
+	if (device)
+		libhal_free_string (device);
+
+	return res;
+}
+
+static void
+hal_device_added (LibHalContext *ctx,
+                  const char    *udi)
+{
+	NautilusBurnDrive *drive;
+
+	drive = libhal_ctx_get_user_data (ctx);
+
+	g_return_if_fail (drive != NULL);
+	g_return_if_fail (udi != NULL);
+
+	if (hal_udi_is_our_drive (ctx, udi, drive)) {
+		g_free (drive->priv->monitor_udi);
+		drive->priv->monitor_udi = g_strdup (udi);
+
+		g_signal_emit (drive, nautilus_burn_drive_table_signals [MEDIA_ADDED], 0);
+	}
+}
+
+static void
+hal_device_removed (LibHalContext *ctx,
+                    const char    *udi)
+{
+	NautilusBurnDrive *drive;
+
+	drive = libhal_ctx_get_user_data (ctx);
+
+	g_return_if_fail (drive != NULL);
+	g_return_if_fail (udi != NULL);
+
+	if (drive->priv->monitor_udi
+	    && strcmp (drive->priv->monitor_udi, udi) == 0) {
+		g_free (drive->priv->monitor_udi);
+		drive->priv->monitor_udi = NULL;
+
+		g_signal_emit (drive, nautilus_burn_drive_table_signals [MEDIA_REMOVED], 0);
+	}
+}
+
+static dbus_bool_t 
+hal_mainloop_integration (LibHalContext *ctx,
+			  DBusError     *error)
+{
+	DBusConnection *dbus_connection;
+
+	dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, error);
+
+	if (dbus_error_is_set (error))
+		return FALSE;
+	
+	dbus_connection_setup_with_g_main (dbus_connection, NULL);
+
+	libhal_ctx_set_dbus_connection (ctx, dbus_connection);
+	
+	return TRUE;
+}
+
+static void
+drive_hal_shutdown (NautilusBurnDrive *drive)
+{
+	DBusError error;
+
+	dbus_error_init (&error);
+	if (! libhal_ctx_shutdown (drive->priv->ctx, &error)) {
+		g_warning ("hal_shutdown failed: %s\n", error.message);
+		dbus_error_free (&error);
+		return;
+	}
+
+	if (! libhal_ctx_free (drive->priv->ctx)) {
+		g_warning ("hal_shutdown failed - unable to free hal context\n");
+	}
+}
+
+static void
+drive_hal_init (NautilusBurnDrive *drive)
+{
+	LibHalContext *ctx;
+	DBusError      error;
+	
+	if (! (ctx = libhal_ctx_new ())) {
+		g_warning ("failed to initialize HAL!");
+		return;
+	}
+	
+	dbus_error_init (&error);
+	if (! hal_mainloop_integration (ctx, &error)) {
+		g_warning ("hal_initialize failed: %s", error.message);
+		dbus_error_free (&error);
+		return;
+	}
+	
+	if (! libhal_ctx_init (ctx, &error)) {
+		g_warning ("hal_initialize failed: %s", error.message);
+		dbus_error_free (&error);
+		libhal_ctx_free (ctx);
+		return;
+	}
+
+	drive->priv->ctx = ctx;
+}
+
 #define LIBHAL_PROP_EXTRACT_BEGIN if (FALSE)
 #define LIBHAL_PROP_EXTRACT_END ;
 #define LIBHAL_PROP_EXTRACT_INT(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_INT32) _where_ = libhal_psi_get_int (&it)
@@ -1296,19 +1453,6 @@ nautilus_burn_drive_eject (NautilusBurnDrive *drive)
 #define LIBHAL_PROP_EXTRACT_BOOL_BITFIELD(_property_, _where_, _field_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ |= libhal_psi_get_bool (&it) ? _field_ : 0
 #define LIBHAL_PROP_EXTRACT_STRLIST(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_STRLIST) _where_ = my_strvdup (libhal_psi_get_strlist (&it))
 
-/** Checks if LibHalContext *ctx == NULL */
-#ifndef LIBHAL_CHECK_LIBHALCONTEXT
-#define LIBHAL_CHECK_LIBHALCONTEXT(_ctx_, _ret_)					\
-	do {									\
-		if (_ctx_ == NULL) {						\
-			fprintf (stderr,					\
-				 "%s %d : LibHalContext *ctx is NULL\n", 	\
-				 __FILE__, __LINE__);				\
-			return _ret_;						\
-		}								\
-	} while(0)
-#endif
-
 static NautilusBurnDrive *
 hal_drive_from_udi (LibHalContext *ctx,
 		    const char    *udi)
@@ -1381,10 +1525,11 @@ hal_scan (gboolean recorder_only)
 	GList         *drives = NULL;
 	int            i;
 	int            num_devices;
-	char**         device_names;
 	LibHalContext *ctx;
+	char         **device_names;
 
 	ctx = get_hal_context ();
+
 	if (ctx == NULL) {
 		return NULL;
 	}
@@ -1405,7 +1550,7 @@ hal_scan (gboolean recorder_only)
 		add_whitelist (drive);
 
 		if (recorder_only && !(drive->type & NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER)) {
-			nautilus_burn_drive_free (drive);
+			nautilus_burn_drive_unref (drive);
 		} else {
 			drives = g_list_prepend (drives, drive);
 		}
@@ -2085,7 +2230,7 @@ freebsd_scan (gboolean recorder_only)
 
 			drives_list = g_list_append (drives_list, drive);
 		} else {
-		    	nautilus_burn_drive_free (drive);
+		    	nautilus_burn_drive_unref (drive);
 		}
 
 		g_free (cam_path);
@@ -2188,7 +2333,7 @@ cdrecord_scan (gboolean recorder_only)
 
 			drives_list = g_list_append (drives_list, drive);
 		} else {
-		    	nautilus_burn_drive_free (drive);
+		    	nautilus_burn_drive_unref (drive);
 		}
 	}
 
@@ -2279,26 +2424,59 @@ nautilus_burn_drive_get_list (gboolean recorder_only,
 	return drives;
 }
 
+/**
+ * nautilus_burn_drive_unref:
+ * @drive: #NautilusBurnDrive
+ *
+ * Decrement the refcount of @drive.
+ *
+ * Since: 2.14
+ **/
+void
+nautilus_burn_drive_unref (NautilusBurnDrive *drive)
+{
+	if (drive == NULL) {
+		return;
+	}
+	
+	g_object_unref (drive);
+}
+
+/**
+ * nautilus_burn_drive_ref:
+ * @drive: #NautilusBurnDrive
+ *
+ * Increment the refcount of @drive.
+ *
+ * Return value: #NautilusBurnDrive
+ *
+ * Since: 2.14
+ **/
+NautilusBurnDrive *
+nautilus_burn_drive_ref (NautilusBurnDrive *drive)
+{
+	if (drive == NULL) {
+		return NULL;
+	}
+	
+	g_object_ref (drive);
+
+	return drive;
+}
+
 /**
  * nautilus_burn_drive_free:
  * @drive: #NautilusBurnDrive to be freed
  *
  * Free @drive.
+ *
+ * Deprecated: Use nautilus_burn_drive_unref() instead
  **/
 void
 nautilus_burn_drive_free (NautilusBurnDrive *drive)
 {
-	g_return_if_fail (drive != NULL);
-
-	if (drive->priv) {
-		g_free (drive->priv->udi);
-		g_free (drive->priv);
-	}
-
-	g_free (drive->display_name);
-	g_free (drive->cdrecord_id);
-	g_free (drive->device);
-	g_free (drive);
+	g_warning ("nautilus_burn_drive_free is deprecated please use nautilus_burn_drive_unref() instead");
+	nautilus_burn_drive_unref (drive);
 }
 
 /**
@@ -2331,13 +2509,14 @@ nautilus_burn_drive_lock (NautilusBurnDrive *drive,
 		DBusError      error;
 		
 		dbus_error_init (&error);
-		ctx = get_hal_context ();
+		ctx = drive->priv->ctx;
+
 		if (ctx != NULL) {
 			res = libhal_device_lock (ctx, 
-					       drive->priv->udi,
-					       reason,
-					       &dbus_reason,
-					       &error);
+						  drive->priv->udi,
+						  reason,
+						  &dbus_reason,
+						  &error);
 
 			if (dbus_error_is_set (&error))
 				dbus_error_free (&error);
@@ -2375,7 +2554,8 @@ nautilus_burn_drive_unlock (NautilusBurnDrive *drive)
 		DBusError      error;
 
 		dbus_error_init (&error);
-		ctx = get_hal_context ();
+		ctx = drive->priv->ctx;
+
 		if (ctx != NULL) {
 			res = libhal_device_unlock (ctx, 
 						    drive->priv->udi,
@@ -2389,6 +2569,49 @@ nautilus_burn_drive_unlock (NautilusBurnDrive *drive)
 	return res;
 }
 
+static void
+nautilus_burn_drive_finalize (GObject *object)
+{
+	NautilusBurnDrive *drive = NAUTILUS_BURN_DRIVE (object);
+
+	g_return_if_fail (object != NULL);
+
+#ifdef USE_HAL
+	drive_hal_shutdown (drive);
+#endif
+
+	if (drive->priv->poll_id > 0)
+		g_source_remove (drive->priv->poll_id);
+
+	g_free (drive->priv->udi);
+	g_free (drive->priv->monitor_udi);
+
+	g_free (drive->display_name);
+	g_free (drive->cdrecord_id);
+	g_free (drive->device);
+
+	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
+		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	}
+}
+
+static void
+nautilus_burn_drive_init (NautilusBurnDrive *drive)
+{
+	drive->priv = NAUTILUS_BURN_DRIVE_GET_PRIVATE (drive);
+
+	drive->type            = 0;
+	drive->display_name    = NULL;
+	drive->max_speed_write = 0;
+	drive->max_speed_read  = 0;
+	drive->cdrecord_id     = NULL;
+	drive->device          = NULL;
+
+#ifdef USE_HAL
+	drive_hal_init (drive);
+#endif
+}
+
 /**
  * nautilus_burn_drive_new:
  *
@@ -2401,11 +2624,7 @@ nautilus_burn_drive_unlock (NautilusBurnDrive *drive)
 NautilusBurnDrive *
 nautilus_burn_drive_new (void)
 {
-	NautilusBurnDrive *drive;
-
-	drive = g_new0 (NautilusBurnDrive, 1);
-	drive->priv = g_new0 (NautilusBurnDrivePriv, 1);
-	return drive;
+	return g_object_new (NAUTILUS_BURN_TYPE_DRIVE, NULL);
 }
 
 /**
@@ -2431,11 +2650,11 @@ nautilus_burn_drive_new_from_path (const char *device)
         for (l = drives; l != NULL; l = l->next) {
 		NautilusBurnDrive *d = l->data;
 		if (g_str_equal (device, d->device)) {
-			drive = nautilus_burn_drive_copy (d);
+			drive = nautilus_burn_drive_ref (d);
 		}
 	}
 
-	g_list_foreach (drives, (GFunc)nautilus_burn_drive_free, NULL);
+	g_list_foreach (drives, (GFunc)nautilus_burn_drive_unref, NULL);
 	g_list_free (drives);
 
 	return drive;
@@ -2448,23 +2667,16 @@ nautilus_burn_drive_new_from_path (const char *device)
  * Return vale: Copy of specified #NautilusBurnDrive.
  *
  * Since: 2.8
+ *
+ * Deprecated: Use nautilus_burn_drive_ref() instead
+ *
  **/
 NautilusBurnDrive *
 nautilus_burn_drive_copy (NautilusBurnDrive *drive)
 {
-	NautilusBurnDrive *drive_copy;
+	g_warning ("nautilus_burn_drive_copy is deprecated please use nautilus_burn_drive_ref() instead");
 
-	drive_copy = nautilus_burn_drive_new ();
-	drive_copy->type = drive->type;
-	drive_copy->display_name = g_strdup (drive->display_name);
-	drive_copy->max_speed_write = drive->max_speed_write;
-	drive_copy->max_speed_read = drive->max_speed_read;
-	drive_copy->cdrecord_id = g_strdup (drive->cdrecord_id);
-	drive_copy->device = g_strdup (drive->device);
-	drive_copy->priv->protocol = drive->priv->protocol;
-	drive_copy->priv->udi = g_strdup (drive->priv->udi);
-
-	return drive_copy;
+	return nautilus_burn_drive_ref (drive);
 }
 
 /**
@@ -2496,6 +2708,60 @@ nautilus_burn_drive_equal (NautilusBurnDrive *a,
 	return strcmp (a->device, b->device) == 0;
 }
 
+static void
+add_desc (GString    *string,
+	  const char *addition)
+{
+	if (strcmp (string->str, "") == 0) {
+		g_string_append_printf (string, "%s", addition);
+	} else {
+		g_string_append_printf (string, ", %s", addition);
+	}
+}
+
+#define NCBD_ADD_TYPE_DESC(_t_, _desc_) if (type & _t_) add_desc (string, _desc_)
+
+/**
+ * nautilus_burn_drive_get_supported_media_string:
+ * @drive: A #NautilusBurnDrive
+ * @writable_only: Set to %TRUE if only writable media should be displayed
+ *
+ * Get a string description of the supported media types.  The
+ * returned string should be freed when no longer needed.
+ *
+ * Returns: a string description of the supported media types
+ *
+ * Since: 2.14
+ **/
+char *
+nautilus_burn_drive_get_supported_media_string (NautilusBurnDrive *drive,
+						gboolean           writable_only)
+{
+	GString *string;
+	int      type;
+
+	type = drive->type;
+
+	string = g_string_new (NULL);
+
+	NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_FILE, "File");
+
+	NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER, "CD-R");
+	NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER, "CD-RW");
+	NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER, "DVD-RAM");
+	NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER, "DVD-RW");
+	NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER, "DVD+R");
+	NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_DL_RECORDER, "DVD+R DL");
+	NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER, "DVD+RW");
+
+	if (! writable_only) {
+		NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_CD_DRIVE, "CD");
+		NCBD_ADD_TYPE_DESC (NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE, "DVD");
+	}
+
+	return g_string_free (string, FALSE);
+}
+
 /**
  * nautilus_burn_drive_media_type_get_string:
  * @type: A #NautilusBurnMediaType
@@ -2542,3 +2808,208 @@ nautilus_burn_drive_media_type_get_string (NautilusBurnMediaType type)
 
 	return _("Broken media type");
 }
+
+#ifdef USE_HAL
+static void
+set_monitor_enabled_hal (NautilusBurnDrive *drive,
+			 gboolean           enabled)
+{
+
+	g_free (drive->priv->monitor_udi);
+	drive->priv->monitor_udi = NULL;
+
+	if (enabled) {
+		char    **device_names;
+		int       num_devices = -1;
+		DBusError error;
+
+		libhal_ctx_set_user_data (drive->priv->ctx, drive);
+		libhal_ctx_set_device_added (drive->priv->ctx, hal_device_added);
+		libhal_ctx_set_device_removed (drive->priv->ctx, hal_device_removed);
+
+		dbus_error_init (&error);
+		device_names = libhal_manager_find_device_string_match (drive->priv->ctx, 
+									"info.parent",
+									drive->priv->udi,
+									&num_devices,
+									&error);
+		
+		if (dbus_error_is_set (&error)) {
+			g_warning ("%s\n", error.message);
+			dbus_error_free (&error);
+		}
+
+		if (num_devices > 0) {
+			drive->priv->monitor_udi = g_strdup (device_names [0]);
+		}
+
+		libhal_free_string_array (device_names);
+	} else {
+		libhal_ctx_set_user_data (drive->priv->ctx, NULL);
+		libhal_ctx_set_device_added (drive->priv->ctx, NULL);
+		libhal_ctx_set_device_removed (drive->priv->ctx, NULL);
+	}
+
+	/* FIXME: check for devices already in the drive and emit signals for them? */
+}
+#else
+
+static gboolean
+poll_tray_opened (NautilusBurnDrive *drive)
+{
+	gboolean is_open;
+
+	is_open = nautilus_burn_drive_door_is_open (drive);
+
+	/* check for no change */
+	if (is_open == drive->priv->monitor_tray_open)
+		return TRUE;
+
+	if (is_open) {
+		/* Tray is now open */
+
+		/* see if we lost media */
+		if (drive->priv->monitor_media_type != NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
+			drive->priv->monitor_media_type = NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+			g_signal_emit (drive, nautilus_burn_drive_table_signals [MEDIA_REMOVED], 0);
+		}
+	} else {
+		NautilusBurnDriveType type;
+
+		/* Tray is now closed */
+
+		/* see if there is now media in the drive */
+		type = nautilus_burn_drive_get_media_type (drive);
+
+		if (type != NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
+			drive->priv->monitor_media_type = type;
+			g_signal_emit (drive, nautilus_burn_drive_table_signals [MEDIA_ADDED], 0);
+		}
+	}
+
+	drive->priv->monitor_tray_open = is_open;
+
+	return TRUE;
+}
+
+static void
+set_monitor_enabled_poll (NautilusBurnDrive *drive,
+			  gboolean           enabled)
+{
+	if (drive->priv->poll_id > 0)
+		g_source_remove (drive->priv->poll_id);
+
+	if (enabled) {
+		drive->priv->monitor_media_type = NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+		drive->priv->monitor_tray_open = FALSE;
+		drive->priv->poll_id = g_timeout_add (2000, (GSourceFunc)poll_tray_opened, drive);
+	} else {
+		drive->priv->monitor_media_type = NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+		drive->priv->monitor_tray_open = FALSE;
+		drive->priv->poll_id = 0;
+	}
+}
+
+#endif /* USE_HAL */
+
+static void
+nautilus_burn_drive_set_monitor_enabled (NautilusBurnDrive *drive,
+					 gboolean           enabled)
+{
+	g_return_if_fail (drive != NULL);
+
+        if (drive->priv->monitor_enabled != enabled) {
+		drive->priv->monitor_enabled = enabled;
+
+#ifdef USE_HAL
+		set_monitor_enabled_hal (drive, enabled);
+#else
+		set_monitor_enabled_poll (drive, enabled);
+#endif
+	}
+}
+
+static void
+nautilus_burn_drive_set_property (GObject	     *object,
+				  guint		      prop_id,
+				  const GValue	     *value,
+				  GParamSpec	     *pspec)
+{
+	NautilusBurnDrive *self;
+
+	self = NAUTILUS_BURN_DRIVE (object);
+
+	switch (prop_id) {
+	case PROP_MONITOR_ENABLED:
+		nautilus_burn_drive_set_monitor_enabled (self, g_value_get_boolean (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+nautilus_burn_drive_get_property (GObject	     *object,
+				  guint		      prop_id,
+				  GValue	     *value,
+				  GParamSpec	     *pspec)
+{
+	NautilusBurnDrive *self;
+
+	self = NAUTILUS_BURN_DRIVE (object);
+
+	switch (prop_id) {
+	case PROP_MONITOR_ENABLED:
+		g_value_set_boolean (value, self->priv->monitor_enabled);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+nautilus_burn_drive_class_init (NautilusBurnDriveClass *klass)
+{
+	GObjectClass *object_class;
+
+	object_class = (GObjectClass *) klass;
+
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = nautilus_burn_drive_finalize;
+	object_class->get_property = nautilus_burn_drive_get_property;
+	object_class->set_property = nautilus_burn_drive_set_property;
+
+	g_type_class_add_private (klass, sizeof (NautilusBurnDrivePrivate));
+
+	/* Properties */
+        g_object_class_install_property (object_class,
+                                         PROP_MONITOR_ENABLED,
+                                         g_param_spec_boolean ("enable-monitor",
+                                                               NULL,
+                                                               NULL,
+                                                               FALSE,
+                                                               G_PARAM_READWRITE));
+
+	/* Signals */
+	nautilus_burn_drive_table_signals [MEDIA_ADDED] =
+		g_signal_new ("media-added",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (NautilusBurnDriveClass,
+					       media_added),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+	nautilus_burn_drive_table_signals [MEDIA_REMOVED] =
+		g_signal_new ("media-removed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (NautilusBurnDriveClass,
+					       media_removed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+}
