@@ -28,6 +28,7 @@
 #ifndef NAUTILUS_BURN_DRIVE_H
 #define NAUTILUS_BURN_DRIVE_H
 
+#include <glib-object.h>
 #include <glib.h>
 
 G_BEGIN_DECLS
@@ -66,20 +67,40 @@ typedef enum {
 	NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_DL_RECORDER = 1 << 9,
 } NautilusBurnDriveType;
 
-typedef struct NautilusBurnDrivePriv NautilusBurnDrivePriv;
+#define NAUTILUS_BURN_TYPE_DRIVE            (nautilus_burn_drive_get_type ())
+#define NAUTILUS_BURN_DRIVE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj),  NAUTILUS_BURN_TYPE_DRIVE, NautilusBurnDrive))
+#define NAUTILUS_BURN_DRIVE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),   NAUTILUS_BURN_TYPE_DRIVE, NautilusBurnDriveClass))
+#define NAUTILUS_BURN_IS_DRIVE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj),  NAUTILUS_BURN_TYPE_DRIVE))
+#define NAUTILUS_BURN_IS_DRIVE_CLASS(klass) (G_TYPE_INSTANCE_GET_CLASS ((klass), NAUTILUS_BURN_TYPE_DRIVE))
 
-typedef struct {
-	NautilusBurnDriveType  type;
-	char                  *display_name;
-	int                    max_speed_write;
-	int                    max_speed_read;
-	char                  *cdrecord_id;
-	char                  *device;
-	NautilusBurnDrivePriv *priv;
-} NautilusBurnDrive;
+typedef struct NautilusBurnDrive                   NautilusBurnDrive;
+typedef struct NautilusBurnDriveClass              NautilusBurnDriveClass;
+typedef struct NautilusBurnDrivePrivate            NautilusBurnDrivePrivate;
+
+struct NautilusBurnDrive {
+	GObject                  parent;
+
+	NautilusBurnDriveType    type;
+	char                    *display_name;
+	int                      max_speed_write;
+	int                      max_speed_read;
+	char                    *cdrecord_id;
+	char                    *device;
+
+	NautilusBurnDrivePrivate *priv;
+};
+
+struct NautilusBurnDriveClass {
+	GObjectClass parent_class;
+
+	void     (*media_added)   (NautilusBurnDrive *drive);
+	void     (*media_removed) (NautilusBurnDrive *drive);
+};
 
 #define NAUTILUS_BURN_DRIVE_SIZE_TO_TIME(size) (size > 0 ? (int) (((size / 1024 / 1024) - 1) * 48 / 7): 0)
 
+GType                 nautilus_burn_drive_get_type                 (void);
+
 GList *               nautilus_burn_drive_get_list                 (gboolean                 recorder_only,
 								    gboolean                 add_image);
 
@@ -87,12 +108,12 @@ NautilusBurnDrive *   nautilus_burn_drive_new                      (void);
 NautilusBurnDrive *   nautilus_burn_drive_new_from_path            (const char              *device_path);
 NautilusBurnDrive *   nautilus_burn_drive_get_file_image           (void);
 
-void                  nautilus_burn_drive_free                     (NautilusBurnDrive       *drive);
+NautilusBurnDrive *   nautilus_burn_drive_ref                      (NautilusBurnDrive       *drive);
+void                  nautilus_burn_drive_unref                    (NautilusBurnDrive       *drive);
 
 gboolean              nautilus_burn_drive_equal                    (NautilusBurnDrive       *a,
 								    NautilusBurnDrive       *b);
 
-NautilusBurnDrive *   nautilus_burn_drive_copy                     (NautilusBurnDrive       *drive);
 NautilusBurnMediaType nautilus_burn_drive_get_media_type           (NautilusBurnDrive       *drive);
 
 NautilusBurnMediaType nautilus_burn_drive_get_media_type_full      (NautilusBurnDrive       *drive,
@@ -112,10 +133,14 @@ gboolean              nautilus_burn_drive_eject                    (NautilusBurn
 gboolean              nautilus_burn_drive_door_is_open             (NautilusBurnDrive       *drive);
 
 const char *          nautilus_burn_drive_media_type_get_string    (NautilusBurnMediaType    type);
+char *                nautilus_burn_drive_get_supported_media_string (NautilusBurnDrive       *drive,
+								      gboolean                 writable_only);
 
 /* The following are deprecated */
 NautilusBurnMediaType nautilus_burn_drive_get_media_type_from_path (const char              *device_path);
 gint64                nautilus_burn_drive_get_media_size_from_path (const char              *device_path);
+NautilusBurnDrive *   nautilus_burn_drive_copy                     (NautilusBurnDrive       *drive);
+void                  nautilus_burn_drive_free                     (NautilusBurnDrive       *drive);
 
 G_END_DECLS
 
