@@ -94,6 +94,8 @@ struct _NautilusBurnProcessFuncs {
 			      gpointer             data);
 };
 
+#define NAUTILUS_BURN_RECORDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NAUTILUS_BURN_TYPE_RECORDER, NautilusBurnRecorderPrivate))
+
 struct NautilusBurnRecorderPrivate {
 	NautilusBurnProcess *process;
 
@@ -235,6 +237,7 @@ can_burn_dvds (NautilusBurnDrive *recorder)
 {
 	if (!(recorder->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER) &&
 	    !(recorder->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER) &&
+	    !(recorder->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_DL_RECORDER) &&
 	    !(recorder->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER)) {
 		return FALSE;
 	}
@@ -326,7 +329,7 @@ compute_time_remaining (gint64 bytes,
 {
 	long secs;
 
-	if (bytes_per_sec <= 0)
+	if (bytes_per_sec <= 1)
 		return -1;
 
 	secs = bytes / bytes_per_sec;
@@ -355,7 +358,8 @@ growisofs_stdout_line (NautilusBurnProcess   *process,
 		if (!process->changed_text) {
 			g_signal_emit (recorder,
 				       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-				       NAUTILUS_BURN_RECORDER_ACTION_WRITING, NAUTILUS_BURN_RECORDER_MEDIA_DVD);
+				       NAUTILUS_BURN_RECORDER_ACTION_WRITING,
+				       NAUTILUS_BURN_RECORDER_MEDIA_DVD);
 		}
 
 		percent = (perc_1 + ((float) perc_2 / 10)) / 100;
@@ -392,7 +396,8 @@ growisofs_blank_stdout_line (NautilusBurnProcess   *process,
 		if (!process->changed_text) {
 			g_signal_emit (recorder,
 				       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-				       NAUTILUS_BURN_RECORDER_ACTION_BLANKING, NAUTILUS_BURN_RECORDER_MEDIA_DVD);
+				       NAUTILUS_BURN_RECORDER_ACTION_BLANKING,
+				       NAUTILUS_BURN_RECORDER_MEDIA_DVD);
 		}
 
 		percent = (perc_1 + ((float) perc_2 / 10)) / 100;
@@ -450,7 +455,8 @@ cdrecord_stdout_line (NautilusBurnProcess   *process,
 		if (!process->changed_text) {
 			g_signal_emit (recorder,
 				       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-				       NAUTILUS_BURN_RECORDER_ACTION_WRITING, NAUTILUS_BURN_RECORDER_MEDIA_CD);
+				       NAUTILUS_BURN_RECORDER_ACTION_WRITING,
+				       NAUTILUS_BURN_RECORDER_MEDIA_CD);
 		}
 
 		if (recorder->priv->tracks_total_bytes > 0) {
@@ -487,7 +493,8 @@ cdrecord_stdout_line (NautilusBurnProcess   *process,
 	} else if (g_str_has_prefix (line, "Fixating...")) {
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-			       NAUTILUS_BURN_RECORDER_ACTION_FIXATING, NAUTILUS_BURN_RECORDER_MEDIA_CD);
+			       NAUTILUS_BURN_RECORDER_ACTION_FIXATING,
+			       NAUTILUS_BURN_RECORDER_MEDIA_CD);
 	} else if (g_str_has_prefix (line, "Fixating time:")) {
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
@@ -498,7 +505,8 @@ cdrecord_stdout_line (NautilusBurnProcess   *process,
 	} else if (g_str_has_prefix (line, "Blanking PMA, TOC, pregap")) {
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-			       NAUTILUS_BURN_RECORDER_ACTION_BLANKING, NAUTILUS_BURN_RECORDER_MEDIA_CD);
+			       NAUTILUS_BURN_RECORDER_ACTION_BLANKING,
+			       NAUTILUS_BURN_RECORDER_MEDIA_CD);
 	}
 
 	return TRUE;
@@ -538,7 +546,8 @@ cdrecord_blank_stdout_line (NautilusBurnProcess   *process,
 	} else if (g_str_has_prefix (line, "Blanking PMA, TOC, pregap")) {
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-			       NAUTILUS_BURN_RECORDER_ACTION_BLANKING, NAUTILUS_BURN_RECORDER_MEDIA_CD);
+			       NAUTILUS_BURN_RECORDER_ACTION_BLANKING,
+			       NAUTILUS_BURN_RECORDER_MEDIA_CD);
 	}
 
 	return TRUE;
@@ -626,7 +635,8 @@ growisofs_stderr_line (NautilusBurnProcess   *process,
 	} else if (strstr (line, "flushing cache") != NULL) {
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-			       NAUTILUS_BURN_RECORDER_ACTION_FIXATING, NAUTILUS_BURN_RECORDER_MEDIA_DVD);
+			       NAUTILUS_BURN_RECORDER_ACTION_FIXATING,
+			       NAUTILUS_BURN_RECORDER_MEDIA_DVD);
 		process->result = NAUTILUS_BURN_RECORDER_RESULT_FINISHED;
 	} else if (strstr (line, ":-(") != NULL || strstr (line, "FATAL") != NULL) {
 		process->last_error = g_strdup (_("Unhandled error, aborting"));
@@ -1410,7 +1420,8 @@ cdrdao_stderr_line (NautilusBurnProcess   *process,
 
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-			       NAUTILUS_BURN_RECORDER_ACTION_WRITING, NAUTILUS_BURN_RECORDER_MEDIA_CD);
+			       NAUTILUS_BURN_RECORDER_ACTION_WRITING,
+			       NAUTILUS_BURN_RECORDER_MEDIA_CD);
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
 			       fraction, secs);
@@ -1422,7 +1433,8 @@ cdrdao_stderr_line (NautilusBurnProcess   *process,
 		process->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [ACTION_CHANGED], 0,
-			       NAUTILUS_BURN_RECORDER_ACTION_WRITING, NAUTILUS_BURN_RECORDER_MEDIA_CD);
+			       NAUTILUS_BURN_RECORDER_ACTION_WRITING,
+			       NAUTILUS_BURN_RECORDER_MEDIA_CD);
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
 			       0.0, -1);
@@ -1821,9 +1833,6 @@ nautilus_burn_recorder_finalize (GObject *object)
 		nautilus_burn_process_free (recorder->priv->process);
 	}
 
-	g_free (recorder->priv);
-	recorder->priv = NULL;
-
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 	}
@@ -1832,7 +1841,7 @@ nautilus_burn_recorder_finalize (GObject *object)
 static void
 nautilus_burn_recorder_init (NautilusBurnRecorder *recorder)
 {
-	recorder->priv = g_new0 (NautilusBurnRecorderPrivate, 1);
+	recorder->priv = NAUTILUS_BURN_RECORDER_GET_PRIVATE (recorder);
 }
 
 /**
@@ -1859,6 +1868,8 @@ nautilus_burn_recorder_class_init (NautilusBurnRecorderClass *klass)
 
 	object_class->finalize = nautilus_burn_recorder_finalize;
 
+	g_type_class_add_private (klass, sizeof (NautilusBurnRecorderPrivate));
+
 	/* Signals */
 	nautilus_burn_recorder_table_signals [PROGRESS_CHANGED] =
 		g_signal_new ("progress-changed",
